package backend

import (
	"context"
	"errors"
	"fmt"
	"kpt-pasture/model"
	"kpt-pasture/util"
	"net/http"
	"strings"
	"time"

	"gorm.io/gorm"

	"gitee.com/xuyiping_admin/pkg/xerr"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
)

func (s *StoreEntry) Detail(ctx context.Context, req *pasturePb.SearchEventRequest) (*pasturePb.CowInfoResponse, error) {
	userModel, err := s.GetUserModel(ctx)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	cowInfo := &model.Cow{}
	pref := s.DB.Model(new(model.Cow)).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Where("ear_number = ?", req.EarNumber)

	if req.NeckRingNumber != "" {
		pref.Where("neck_ring_number = ?", req.NeckRingNumber)
	}

	if err = pref.Order("id desc").
		First(cowInfo).Error; err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return nil, xerr.Custom("该牛只未找到")
		} else {
			return nil, xerr.WithStack(err)
		}
	}

	cowTypeMap := s.CowTypeMap()
	breedStatusMap := s.CowBreedStatusMap()
	cowKindMap := s.CowKindMap()
	cowSourceMap := s.CowSourceMap()
	admissionStatusMap := s.AdmissionStatusMap()
	healthStatusMap := s.HealthStatusMap()
	purposeMap := s.PurposeMap()
	systemBasic, err := s.GetSystemBasicByName(ctx, userModel.AppPasture.Id, model.PregnancyAge)
	if err != nil {
		return nil, xerr.Custom("请在基础参数配置妊娠天数")
	}

	cowDetails := model.CowSlice([]*model.Cow{cowInfo}).ToPB(
		cowTypeMap, breedStatusMap, cowKindMap, cowSourceMap,
		admissionStatusMap, healthStatusMap, purposeMap, systemBasic.MinValue,
	)
	if len(cowDetails) != 1 {
		return nil, xerr.Custom("该牛只未找到")
	}

	data := cowDetails[0]
	return &pasturePb.CowInfoResponse{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: data,
	}, nil
}

func (s *StoreEntry) List(ctx context.Context, req *pasturePb.SearchEventRequest, pagination *pasturePb.PaginationModel) (*pasturePb.SearchCowListResponse, error) {
	userModel, err := s.GetUserModel(ctx)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	systemBasic, err := s.GetSystemBasicByName(ctx, userModel.AppPasture.Id, model.PregnancyAge)
	if err != nil {
		return nil, xerr.Custom("请在基础参数配置妊娠天数")
	}

	cowList := make([]*model.Cow, 0)
	var count int64 = 0
	pref := s.DB.Model(new(model.Cow)).
		Where("pasture_id = ?", userModel.AppPasture.Id)

	if len(req.CowId) > 0 {
		cowIds := strings.Split(req.CowId, ",")
		pref.Where("id IN ?", cowIds)
	}

	if req.EarNumber != "" {
		pref.Where("ear_number like ?", fmt.Sprintf("%s%s%s", "%", req.EarNumber, "%"))
	}

	if req.Id > 0 {
		pref.Where("id = ?", req.Id)
	}

	if req.PenId > 0 {
		pref.Where("pen_id = ?", req.PenId)
	}

	if req.CowType > 0 {
		pref.Where("cow_type = ?", req.CowType)
	}

	if req.BreedStatus > 0 {
		pref.Where("breed_status = ?", req.BreedStatus)
	}

	if req.CowKind > 0 {
		pref.Where("cow_kind = ?", req.CowKind)
	}

	if req.Sex > 0 {
		pref.Where("sex = ?", req.Sex)
	}

	if req.Lact > 0 {
		pref.Where("lact = ?", req.Lact)
	}

	if req.CowSource > 0 {
		pref.Where("source_id = ?", req.CowSource)
	}

	if err = pref.Order("id desc").
		Count(&count).
		Limit(int(pagination.PageSize)).
		Offset(int(pagination.PageOffset)).
		Find(&cowList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	cowTypeMap := s.CowTypeMap()
	breedStatusMap := s.CowBreedStatusMap()
	cowKindMap := s.CowKindMap()
	cowSourceMap := s.CowSourceMap()
	admissionStatusMap := s.AdmissionStatusMap()
	healthStatusMap := s.HealthStatusMap()
	purposeMap := s.PurposeMap()
	return &pasturePb.SearchCowListResponse{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &pasturePb.SearchCowData{
			List: model.CowSlice(cowList).ToPB(
				cowTypeMap, breedStatusMap, cowKindMap, cowSourceMap,
				admissionStatusMap, healthStatusMap, purposeMap, systemBasic.MinValue,
			),
			Total:    int32(count),
			PageSize: pagination.PageSize,
			Page:     pagination.Page,
		},
	}, nil
}

func (s *StoreEntry) EventList(ctx context.Context, req *pasturePb.SearchCowEventListRequest, pagination *pasturePb.PaginationModel) (*pasturePb.CowEventListResponse, error) {
	userModel, err := s.GetUserModel(ctx)
	if err != nil {
		return nil, xerr.WithStack(err)
	}
	eventCowLogList := make([]*model.EventCowLog, 0)
	cowInfo, err := s.GetCowEventByEarNumber(ctx, userModel.AppPasture.Id, req.EarNumber)
	if err != nil {
		return nil, xerr.Customf("错误的牛只信息: %s", req.EarNumber)
	}

	eventCowLog := &model.EventCowLog{CowId: cowInfo.Id}
	pref := s.DB.Table(eventCowLog.TableName()).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Where("cow_id = ?", cowInfo.Id)

	if req.Lact >= 0 {
		pref.Where("lact = ?", req.Lact)
	}

	if req.EventCategoryKind > 0 {
		pref.Where("event_type = ?", req.EventCategoryKind)
	}

	if err = pref.Order("id desc").
		Limit(int(pagination.PageSize)).
		Offset(int(pagination.PageOffset)).
		Find(&eventCowLogList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	eventCategoryMap := s.EventCategoryMap()
	return &pasturePb.CowEventListResponse{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &pasturePb.CowEventData{
			List:     model.EventCowLogSlice(eventCowLogList).ToPB(eventCategoryMap),
			Total:    int32(len(eventCowLogList)),
			PageSize: pagination.PageSize,
			Page:     pagination.Page,
		},
	}, nil
}

func (s *StoreEntry) BehaviorCurve(ctx context.Context, req *pasturePb.CowBehaviorCurveRequest) (*model.CowBehaviorCurveResponse, error) {
	userModel, err := s.GetUserModel(ctx)
	if err != nil {
		return nil, xerr.WithStack(err)
	}
	cowInfo, err := s.GetCowEventByEarNumber(ctx, userModel.AppPasture.Id, req.EarNumber)
	if err != nil {
		return nil, xerr.Customf("错误的牛只信息: %d", req.CowId)
	}

	nowTime := time.Now().Local()
	nowDayZero := util.TimeParseLocalUnix(nowTime.Format(model.LayoutDate2))
	endDataTime := nowTime.Format(model.LayoutDate2)
	startDataTime := nowTime.AddDate(0, 0, -30).Format(model.LayoutDate2)

	// 行为曲线数据
	neckActiveHabitList := make([]*model.NeckActiveHabit, 0)
	if err = s.DB.Table(new(model.NeckActiveHabit).TableName()).
		Where("neck_ring_number = ?", cowInfo.NeckRingNumber).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Where("is_show = ?", pasturePb.IsShow_Ok).
		Where("cow_id > ?", 0).
		Where("heat_date BETWEEN ? AND ?", startDataTime, endDataTime).
		Order("heat_date, frameid").
		Find(&neckActiveHabitList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	data := model.NeckActiveHabitSlice(neckActiveHabitList).ToPB(req.CurveName)
	eventMapList := s.EventTypeMap()
	for k, v := range eventMapList {
		if k == pasturePb.EventType_Enter || k == pasturePb.EventType_Body_Score || k == pasturePb.EventType_Birth ||
			k == pasturePb.EventType_Weaning || k == pasturePb.EventType_Sale || k == pasturePb.EventType_Weight ||
			k == pasturePb.EventType_Castrated {
			continue
		}
		data.EventMap[k] = v
	}
	data.LowActivity = 62
	data.MiddleActivity = 80
	// 牛只事件列表
	eventLogList := make([]*model.EventCowLog, 0)
	eventLog := &model.EventCowLog{CowId: cowInfo.Id}
	if err = s.DB.Table(eventLog.TableName()).
		Where("cow_id = ?", cowInfo.Id).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Where("event_at BETWEEN ? AND ?", nowDayZero-(30*86400), nowDayZero+86400).
		Order("event_at").
		Find(&eventLogList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	for _, v := range eventLogList {
		if v.EventType == pasturePb.EventType_Enter || v.EventType == pasturePb.EventType_Body_Score ||
			v.EventType == pasturePb.EventType_Birth || v.EventType == pasturePb.EventType_Weaning ||
			v.EventType == pasturePb.EventType_Sale || v.EventType == pasturePb.EventType_Weight ||
			v.EventType == pasturePb.EventType_Castrated {
			continue
		}
		eventAt := time.Unix(v.EventAt, 0).Local()
		if data.EventList[v.EventTypeName] == nil {
			data.EventList[v.EventTypeName] = make([]string, 0)
		}
		data.EventList[v.EventTypeName] = append(data.EventList[v.EventTypeName], fmt.Sprintf("%s 09", eventAt.Format(model.LayoutDate2)))
	}

	// 发情数据
	estrusList := make([]*model.EventEstrus, 0)
	if err = s.DB.Table(new(model.EventEstrus).TableName()).
		Where("cow_id = ?", cowInfo.Id).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Where("reality_day BETWEEN ? AND ?", startDataTime, endDataTime).
		Order("reality_day").
		Find(&estrusList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	for _, v := range estrusList {
		if data.EstrusList[v.Level] == nil {
			data.EstrusList[v.Level] = make([]string, 0)
		}
		if v.RealityDay > 0 {
			// 格式化为到小时的字符串
			hourStr := time.Unix(v.RealityDay, 0).Local().Format(model.LayoutHour)
			data.EstrusList[v.Level] = append(data.EstrusList[v.Level], hourStr)
		}
	}

	return &model.CowBehaviorCurveResponse{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: data,
	}, nil
}

func (s *StoreEntry) CowGrowthCurve(ctx context.Context, req *pasturePb.CowGrowthCurveRequest) (*pasturePb.CowGrowthCurveResponse, error) {
	userModel, err := s.GetUserModel(ctx)
	if err != nil {
		return nil, xerr.WithStack(err)
	}
	cowInfo, err := s.GetCowInfoByEarNumber(ctx, userModel.AppPasture.Id, req.EarNumber)
	if err != nil {
		return nil, xerr.Customf("错误的牛只信息: %s", req.EarNumber)
	}

	weightList := make([]*model.EventWeight, 0)
	if err = s.DB.Table(new(model.EventWeight).TableName()).
		Where("cow_id = ?", cowInfo.Id).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Order("weight_at").
		Find(&weightList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	eventCowLogList := make([]*model.EventCowLog, 0)
	eventCowLog := &model.EventCowLog{CowId: cowInfo.Id}
	if err = s.DB.Table(eventCowLog.TableName()).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Where("cow_id = ?", cowInfo.Id).
		Order("id desc").
		Find(&eventCowLogList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	return &pasturePb.CowGrowthCurveResponse{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: model.EventWeightSlice(weightList).ToPB(cowInfo, eventCowLogList),
	}, nil
}

func (s *StoreEntry) CowLactCurve(ctx context.Context, req *pasturePb.CowLactCurveRequest) (*pasturePb.CowLactCurveResponse, error) {
	userModel, err := s.GetUserModel(ctx)
	if err != nil {
		return nil, xerr.WithStack(err)
	}
	cowInfo, err := s.GetCowInfoByCowId(ctx, userModel.AppPasture.Id, int64(req.CowId))
	if err != nil {
		return nil, xerr.Customf("错误的牛只信息: %d", req.CowId)
	}

	cowLactList := make([]*model.CowLact, 0)
	if err = s.DB.Table(new(model.CowLact).TableName()).
		Where("cow_id = ?", req.CowId).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Order("lact").
		Find(&cowLactList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	data := &pasturePb.CowLactCurveData{
		DateTime:            make(map[int32]string),
		WeekAvgMilk:         make([]float32, 0),
		DayMilk:             make([]float32, 0),
		DHI:                 make([]float32, 0),
		MilkProductionTrend: make([]float32, 0),
		DayHigh:             make([]int32, 0),
		DayRumina:           make([]int32, 0),
		DayIntake:           make([]int32, 0),
		DayInactive:         make([]int32, 0),
		DayChew:             make([]int32, 0),
		DayImmobility:       make([]int32, 0),
		EstrusWarning:       make(map[int32]int32),
	}

	if len(cowLactList) <= 0 {
		return &pasturePb.CowLactCurveResponse{
			Code: http.StatusOK,
			Msg:  "ok",
			Data: data,
		}, nil
	}
	starTime := ""
	endTime := ""
	cowLactMap := make(map[int32]*model.CowLact)
	for _, v := range cowLactList {
		cowLactMap[v.Lact] = v
		if v.Lact == req.Lact {
			starTime = v.StartTime
		}
	}
	if st, ok := cowLactMap[req.Lact+1]; ok {
		et, _ := util.TimeParseLocal(model.LayoutDate2, st.StartTime)
		endTime = et.AddDate(0, 0, -1).Format(model.LayoutDate2)
	} else {
		endTime = time.Now().Local().Format(model.LayoutDate2)
	}

	neckRingList := make([]*model.NeckActiveHabit, 0)
	if err = s.DB.Model(new(model.NeckActiveHabit)).
		Where("neck_ring_number = ?", cowInfo.NeckRingNumber).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Where("cow_id > ?", 0).
		Where("heat_date BETWEEN ? AND ?", starTime, endTime).
		Order("heat_date, frameid").Find(&neckRingList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	return &pasturePb.CowLactCurveResponse{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: data,
	}, nil
}

func (s *StoreEntry) BehaviorRate(ctx context.Context, req *pasturePb.CowBehaviorRateRequest) (*pasturePb.CowBehaviorRateResponse, error) {
	userModel, err := s.GetUserModel(ctx)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	cowInfo, err := s.GetCowInfoByEarNumber(ctx, userModel.AppPasture.Id, req.EarNumber)
	if err != nil {
		return nil, xerr.Customf("错误的牛只信息: %d", req.EarNumber)
	}

	if req.EndAt <= 0 || req.StartAt <= 0 || req.EndAt < req.StartAt {
		return nil, xerr.Customf("时间范围错误")
	}

	t1 := time.Unix(int64(req.StartAt), 0).Local().Format(model.LayoutDate2)
	t2 := time.Unix(int64(req.EndAt), 0).Local().Format(model.LayoutDate2)

	dataBetween, err := util.GetDaysBetween(t1, t2)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	if cowInfo.NeckRingNumber == "" {
		return nil, xerr.Customf("该牛只未佩戴脖环: %s", req.EarNumber)
	}

	neckActiveHabitList := make([]*model.NeckActiveHabit, 0)
	if err = s.DB.Model(new(model.NeckActiveHabit)).
		Select("heat_date,SUM(rumina) as rumina,SUM(intake) as intake,SUM(inactive) as inactive,SUM(gasp) AS gasp").
		Where("neck_ring_number = ?", cowInfo.NeckRingNumber).
		Where("pasture_id = ?", userModel.AppPasture.Id).
		Where("cow_id > ?", 0).
		Where("heat_date BETWEEN ? AND ?", t1, t2).
		Order("heat_date").
		Group("heat_date").
		Find(&neckActiveHabitList).Error; err != nil {
	}

	return &pasturePb.CowBehaviorRateResponse{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: model.NeckActiveHabitSlice(neckActiveHabitList).ToPB2(dataBetween),
	}, nil
}
