package backend

import (
	"context"
	"fmt"
	"kpt-pasture/model"
	"kpt-pasture/util"
	"net/http"
	"time"

	"gorm.io/gorm"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
	"gitee.com/xuyiping_admin/pkg/xerr"
)

func (s *StoreEntry) EstrusOrAbortionCowList(ctx context.Context, req *pasturePb.WarningItemsRequest, pagination *pasturePb.PaginationModel) (*pasturePb.EstrusResponse, error) {
	userModel, err := s.GetUserModel(ctx)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	var count int64
	neckRingEstrusList := make([]*model.NeckRingEstrusWarning, 0)
	var pref *gorm.DB
	switch req.Kind {
	case "abortion":
		pref, err = s.AbortionWarningQuery(ctx, userModel.AppPasture.Id)
		if err != nil {
			return nil, xerr.WithStack(err)
		}
	default:
		pref, err = s.EstrusWarningQuery(ctx, userModel.AppPasture.Id)
		if err != nil {
			return nil, xerr.WithStack(err)
		}
	}

	if len(req.EarNumber) > 0 {
		pref.Where("a.ear_number = ?", req.EarNumber)
	}

	if req.Level > 0 {
		pref.Where("a.level = ?", req.Level)
	}

	if len(req.PenIds) > 0 {
		pref.Where("b.pen_id IN ?", req.PenIds)
	}

	if err = pref.Order("a.level DESC").
		Count(&count).
		Limit(int(pagination.PageSize)).
		Offset(int(pagination.PageOffset)).
		Find(&neckRingEstrusList).Error; err != nil {
		return nil, xerr.WithStack(err)
	}

	cowMap := make(map[int64]*model.Cow)
	eventLogMap := make(map[int64]string)
	cowIds := make([]int64, 0)
	for _, v := range neckRingEstrusList {
		cowIds = append(cowIds, v.CowId)
		lastEventLog := s.GetCowLastEvent(userModel.AppPasture.Id, v.CowId, pasturePb.EventCategory_Breed)
		if lastEventLog != nil {
			eventLogMap[v.CowId] = lastEventLog.EventDescription
		}
	}

	if len(cowIds) > 0 {
		cowList, _ := s.GetCowInfoByCowIds(ctx, userModel.AppPasture.Id, cowIds)
		for _, cow := range cowList {
			cowMap[cow.Id] = cow
		}
	}

	return &pasturePb.EstrusResponse{
		Code: http.StatusOK,
		Msg:  "ok",
		Data: &pasturePb.EstrusData{
			List:     model.NeckRingEstrusWarningSlice(neckRingEstrusList).ToPB(cowMap, eventLogMap),
			Total:    int32(count),
			PageSize: pagination.PageSize,
			Page:     pagination.Page,
		},
	}, nil
}

func (s *StoreEntry) EstrusWarningQuery(ctx context.Context, pastureId int64) (*gorm.DB, error) {
	nowTime := time.Now().Local()
	startTime := time.Unix(util.TimeParseLocalUnix(nowTime.Format(model.LayoutDate2)), 0).Local().Format(model.LayoutTime)
	entTime := time.Unix(util.TimeParseLocalEndUnix(nowTime.AddDate(0, 0, 1).Local().Format(model.LayoutDate2)), 0).Format(model.LayoutTime)
	systemBasic, err := s.FindSystemBasic(ctx, pastureId, model.EstrusWaringDays)
	if err != nil {
		return nil, xerr.WithStack(err)
	}

	return s.DB.Table(fmt.Sprintf("%s as a", new(model.NeckRingEstrusWarning).TableName())).
		Joins(fmt.Sprintf("JOIN %s AS b on a.cow_id = b.id", new(model.Cow).TableName())).
		Where("b.last_mating_at < UNIX_TIMESTAMP(a.date_time)").
		Where("b.admission_status = ?", pasturePb.AdmissionStatus_Admission).
		Where("b.is_forbidden_mating = ?", pasturePb.IsShow_No).
		Where("a.level >= ?", pasturePb.EstrusLevel_Low).
		Where("a.pasture_id = ?", pastureId).
		Where("a.date_time BETWEEN ? AND ?", startTime, entTime).
		Where(s.DB.Where("b.last_mating_at < UNIX_TIMESTAMP(a.first_time)").
			Or(s.DB.Where("b.last_mating_at = ?", 0).
				Where("b.calving_age > ?", systemBasic.MinValue).
				Or("b.lact = ?", 0))).
		Where("a.is_show = ?", pasturePb.IsShow_Ok), nil
}

func (s *StoreEntry) AbortionWarningQuery(ctx context.Context, pastureId int64) (*gorm.DB, error) {
	return s.DB.Table(fmt.Sprintf("%s as a", new(model.NeckRingEstrusWarning).TableName())).
		Joins(fmt.Sprintf("JOIN %s AS b on a.cow_id = b.id", new(model.Cow).TableName())).
		Where("b.pregnancy_age BETWEEN ? AND ?", 1, 260).
		Where("b.admission_status = ?", pasturePb.AdmissionStatus_Admission).
		Where("b.is_pregnant = ?", pasturePb.IsShow_Ok).
		Where("a.level >= ?", pasturePb.EstrusLevel_Middle).
		Where("a.pasture_id = ?", pastureId).
		Where("a.is_show = ?", pasturePb.IsShow_Ok), nil
}
